a = +1; % advection velocity
tend = 3.; % end time
M = 25; % number of grid points
CFL = 0.95; % CFL safety condition so that within stability
           % (0 <= abs(a)*dt/dx <= CFL <= 1)
x = linspace(0., 1., M + 1); % the grid
dx = x(2) - x(1); % grid spacing
dt = CFL*dx/abs(a); % time step
scheme = 0; % 0 -> upwind a > 0, 1 -> upwind a < 0, 2 -> centered

% initial conditions
u0 = sin(2.*pi*x(1:end-1)); % sine wave
% u0 = exp(- 50.*(x(1:end-1) - 0.5).^2); % Gaussian pulse
% u0 = x(1:end-1) >= 0.25 & x(1:end-1) <= 0.75; % box
u = u0;

% evolve
time = 0.;
while time < tend
  if time + dt > tend
    dt = tend - time; % to reach final time tend exactly
  end
  time = time + dt;
  % the function circshift is useful for the periodic boundary conditions :-)
  switch scheme
    case 0
      % first-order upwind scheme with a > 0
      % u^{n+1}_{i} = u^{n}_{i} - a*dt(u^{n}_{i} - u^{n}_{i-1})/dx
      u = u - a*dt*(u - circshift(u, 1))/dx;
    case 1
      % first-order upwind scheme with a < 0
      % u^{n+1}_{i} = u^{n}_{i} - a*dt(u^{n}_{i+1} - u^{n}_{i})/dx
      u = u - a*dt*(circshift(u, -1) - u)/dx;
    case 2
      % centered finite difference (unstable!)
      u = u - 0.5*a*dt*(circshift(u, -1) - circshift(u, 1))/dx;
    otherwise
      disp('Unknown scheme choice');
      break;
  end
end

% plot
clf;
hold on;
plot(x(1:end-1), u0);
plot(x(1:end-1), u, '-+');
box on;
xlabel('x');
ylabel('u');
title('Linear advection equation');
legend('Initial condition u(x, 0) = u_{0}(x)', ...
       'Numerical solution u(x, tend)')
hold off;
